import os
import os.path
import subprocess

import click
import inquirer
from claudia.python.lib.framework.common import read_env_var
from inquirer.themes import GreenPassion

from claudia.versions import XRPL_PY_VERSION, XRPL_JS_VERSION


class SingletonClass(object):
    def __new__(cls):
        if not hasattr(cls, 'instance'):
            cls.instance = super(SingletonClass, cls).__new__(cls)
        return cls.instance


singleton = SingletonClass()
singleton.PROJECT_ROOT_DIR = os.path.dirname(__file__)


def set_to_js_wd():
    os.chdir("{}/javascript/".format(singleton.PROJECT_ROOT_DIR))


def set_to_py_wd():
    os.chdir("{}/python/".format(singleton.PROJECT_ROOT_DIR))


def set_to_project_root_wd():
    os.chdir(singleton.PROJECT_ROOT_DIR)


def compose_feature_helper_message():
    message = "Feature file to be used for the test run. Allowed values are "
    features = get_feature_file_names()
    for i in range(0, len(features)):
        message += "'{}', ".format(features[i])
    message += "and 'all' and is defaulted to 'payments'. \n\nMore information: https://behave.readthedocs.io/en/latest/tutorial.html?highlight=feature#feature-files."
    return message


def get_feature_file_names():
    set_to_project_root_wd()
    files = os.listdir('./features/')
    features = []
    for i in range(0, len(files)):
        features.append(files[i].replace(".feature", ""))

    return features


@click.group()
@click.version_option(message=f'%(prog)s version %(version)s\n  - Supports:\n    - XRPL-py v{XRPL_PY_VERSION}\n    - XRPL.js v{XRPL_JS_VERSION}')
def main():
    """Claudia says hi! Please choose a command to perform an action. A command can have multiple sub-commands and options. Use '--help' option for more information."""


@main.command()
def ui():
    """Launch Claudia UI"""
    try:
        clear_screen()
        launch_claudia_ui()
    except Exception as e:
        pass


def launch_claudia_ui():
    create_streamlit_credentials()
    set_to_project_root_wd()
    command = "streamlit run ui/claudia_ui.py"
    return subprocess.call(command, shell=True)


def create_streamlit_credentials():
    credentials_file = os.path.expanduser("~/.streamlit/credentials.toml")
    if not os.path.isfile(credentials_file):
        os.makedirs(os.path.dirname(credentials_file), exist_ok=True)
        f = open(credentials_file, 'w')
        f.write('[general]\nemail = ""\n')


@main.command()
def demo():
    """Launch claudia in demo mode"""
    try:
        clear_screen()
        launch_main_menu()
    except Exception as e:
        pass


def launch_main_menu():
    questions = [
        inquirer.List(
            "main_menu",
            message="Welcome to Claudia Demo! Please use ↑ ↓ and ↵ keys to choose an option. Current Selection",
            choices=[
                "Custom XRPL Networks",
                "XRPL Tests",
                "Settings",
                "Launch Claudia UI",
                "Exit"
            ],
        ),
    ]

    selection_text = inquirer.prompt(questions, theme=GreenPassion())['main_menu']

    if selection_text == "Custom XRPL Networks":
        clear_screen()
        launch_custom_rippled_networks_menu()
    elif selection_text == "XRPL Tests":
        clear_screen()
        launch_test_task_menu()
    elif selection_text == "Settings":
        clear_screen()
        launch_settings_menu()
    elif selection_text == "Launch Claudia UI":
        clear_screen()
        launch_claudia_ui()
    elif selection_text == 'Exit':
        print("Thank you for using Claudia demo. Bye!")
        return


def launch_custom_rippled_networks_menu():
    relaunch_wizard = True
    questions = [
        inquirer.List(
            "rippled_task_menu",
            message="Here you can find a list of rippled related tasks. Please use ↑ ↓ and ↵ keys to choose an option. Current Selection",
            choices=[
                "Build rippled from local code",
                "Install rippled",
                "Start local-mainnet",
                "Stop local-mainnet",
                "Check local-mainnet status",
                "Build witness server",
                "Start local-sidechain",
                "Stop local-sidechain",
                "Check witness server status",
                "Back to main menu"
            ],
        ),
    ]

    selection_text = inquirer.prompt(questions, theme=GreenPassion())['rippled_task_menu']
    if selection_text == "Build rippled from local code":
        relaunch_wizard = False
        repo_path = get_valid_repo_path()
        if repo_path == "" or repo_path.replace("'", "").replace('"', '').lower() == 'back':
            relaunch_wizard = True
        else:
            clear_screen()
            print("\nINFO: In order to '{}', run: 'claudia rippled build --repo <absolute_path_to_local_repo>'".format(
                selection_text))
            print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
            launch_command_rippled_build(repo_path)
    elif selection_text == 'Install rippled':
        relaunch_wizard = False
        clear_screen()
        rippled_branch = get_rippled_branch()
        if rippled_branch == "" or rippled_branch.replace("'", "").replace('"', '').lower() == 'back':
            relaunch_wizard = True
        else:
            clear_screen()
            print("\nINFO: In order to '{}', run: 'claudia rippled install --rippled_branch <branch_name>'".format(
                selection_text))
            print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
            launch_command_rippled_install(rippled_branch)
    elif selection_text == 'Start local-mainnet':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia local-mainnet start'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_network_start()
    elif selection_text == 'Check local-mainnet status':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia local-mainnet status'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_network_status()
    elif selection_text == 'Stop local-mainnet':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia local-mainnet stop'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_network_stop()
    elif selection_text == 'Build witness server':
        relaunch_wizard = False
        repo_path = get_valid_repo_path()
        if repo_path == "" or repo_path.replace("'", "").replace('"', '').lower() == 'back':
            relaunch_wizard = True
        else:
            clear_screen()
            print("\nINFO: In order to '{}', run: 'claudia witness build --repo <absolute_path_to_local_repo>'".format(
                selection_text))
            print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
            launch_command_witness_build(repo_path)
    elif selection_text == 'Check witness server status':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia witness status'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_witness_status()
    elif selection_text == 'Start local-sidechain':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia local-sidechain start'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_sidechain_start()
    elif selection_text == 'Stop local-sidechain':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia local-sidechain stop'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_sidechain_stop()
    elif selection_text == 'Back to main menu':
        clear_screen()
        launch_main_menu()
        return

    if relaunch_wizard:
        clear_screen()
        launch_custom_rippled_networks_menu()
    else:
        if get_confirmation("Would you like to continue with the demo?"):
            clear_screen()
            launch_custom_rippled_networks_menu()
        else:
            print("Thank you for using Claudia demo. Bye!")
            return


def launch_test_task_menu():
    relaunch_wizard = True
    questions = [
        inquirer.List(
            "test_task_menu",
            message="Here you can find a list of test related tasks. Please use ↑ ↓ and ↵ keys to choose an option. Current Selection",
            choices=[
                "Run system tests",
                "Run unit tests",
                "Back to main menu"
            ],
        ),
    ]

    selection_text = inquirer.prompt(questions, theme=GreenPassion())['test_task_menu']
    if selection_text == 'Run system tests':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia run systemtests'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        client_type = 'websocket'
        invalidate_cache = 'false'
        feature_files = get_feature_file_names()
        feature_files.append('all')
        lib = inquirer.prompt(
            [inquirer.List("confirmation", message="Please choose library type. Current Selection",
                           choices=["py", "js"], default="py")],
            theme=GreenPassion()
        )['confirmation']
        if lib == 'py':
            client_type = inquirer.prompt(
                [inquirer.List("confirmation", message="Please choose client type. Current Selection",
                               choices=["websocket", "jsonrpc"], default="websocket")],
                theme=GreenPassion()
            )['confirmation']
        network = inquirer.prompt(
            [inquirer.List("confirmation", message="Please choose network type. Current Selection",
                           choices=["local-mainnet", "devnet", "testnet", "local-sidechain"],
                           default="local-mainnet")],
            theme=GreenPassion()
        )['confirmation']

        tag = inquirer.prompt(
            [inquirer.List("confirmation", message="Please choose tag. Current Selection",
                           choices=['smoke', 'regression', 'time_intensive'], default="smoke")],
            theme=GreenPassion()
        )['confirmation']

        feature = inquirer.prompt(
            [inquirer.List("confirmation",
                           message="Please choose the feature file. Choose 'all' to include all feature files. Current Selection",
                           choices=feature_files, default="payments")],
            theme=GreenPassion()
        )['confirmation']

        if lib == 'js':
            invalidate_cache = inquirer.prompt(
                [inquirer.List("confirmation",
                               message="Please choose if you would like to destroy cache, if any. Current Selection",
                               choices=["false", "true"], default="false")],
                theme=GreenPassion()
            )['confirmation']

        launch_command_run_system_tests(lib, client_type, network, tag, feature, invalidate_cache)

    elif selection_text == 'Run unit tests':
        relaunch_wizard = False
        print("\nINFO: In order to '{}', run: 'claudia run unittests'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        testname = get_testname()
        if testname.replace("'", "").replace('"', '').lower() == 'back':
            relaunch_wizard = True
        else:
            launch_command_run_unit_tests(testname)
    elif selection_text == 'Back to main menu':
        clear_screen()
        launch_main_menu()
        return

    if relaunch_wizard:
        clear_screen()
        launch_test_task_menu()
    else:
        if get_confirmation("Would you like to continue with the demo?"):
            clear_screen()
            launch_test_task_menu()
        else:
            print("Thank you for using Claudia demo. Bye!")
            return


def launch_command_run_system_tests(lib, client_type, network, tag, feature, invalidate_cache):
    try:
        click.echo("INFO: Navigate to '{}' to view explorer.\n".format(get_explorer_url(network)))
        setup_system_test_env(lib, client_type, network, tag, feature, invalidate_cache)
        print("Setting CONNECTION_SCHEME='{}', CONNECTION_URL='{}' and CONNECTION_TYPE='{}'".format(
            read_env_var('CONNECTION_SCHEME'),
            read_env_var('CONNECTION_URL'),
            read_env_var('CONNECTION_TYPE'))
        )

        command = get_launch_command_system_test_tests(lib, tag, feature)
        subprocess.call(command, shell=True)
        teardown_system_test_env(lib)
    except Exception as e:
        set_to_project_root_wd()
        raise e


def launch_settings_menu():
    relaunch_wizard = True
    questions = [
        inquirer.List(
            "misc_task_menu",
            message="Here you can find a list of all the settings. Please use ↑ ↓ and ↵ keys to choose an option. Current Selection",
            choices=[
                "Set install mode as build",
                "Set install mode as install",
                "Enable a rippled feature",
                "Disable a rippled feature",
                "Clean up the host and free resources",
                "List system requirements (prerequisites)",
                "List system test features",
                "Print already built/installed rippled version",
                "Back to main menu"
            ],
        ),
    ]

    selection_text = inquirer.prompt(questions, theme=GreenPassion())['misc_task_menu']
    if selection_text == 'List system test features':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia list system-test-features'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_system_test_features()
    elif selection_text == 'List system requirements (prerequisites)':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia list system-requirements'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_system_requirements()
    elif selection_text == 'Set install mode as build':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia set-install-mode build'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_set_install_mode_to_build()
    elif selection_text == 'Set install mode as install':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia set-install-mode install'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_set_install_mode_to_install()
    elif selection_text == 'Enable a rippled feature':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia enable-feature --feature <feature_name>'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        feature_name = get_feature_name()
        if feature_name == "" or feature_name.replace("'", "").replace('"', '').lower() == 'back':
            relaunch_wizard = True
        else:
            launch_command_enable_feature(feature_name)
    elif selection_text == 'Disable a rippled feature':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia disable-feature --feature <feature_name>'".format(
            selection_text
        ))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        feature_name = get_feature_name()
        if feature_name == "" or feature_name.replace("'", "").replace('"', '').lower() == 'back':
            relaunch_wizard = True
        else:
            launch_command_disable_feature(feature_name)
    elif selection_text == 'Clean up the host and free resources':
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia clean'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_clean()
    elif selection_text == 'Print already built/installed rippled version'.format(selection_text):
        relaunch_wizard = False
        clear_screen()
        print("\nINFO: In order to '{}', run: 'claudia rippled version'".format(selection_text))
        print("You have opted to execute '{}'. Starting now...\n".format(selection_text))
        launch_command_rippled_version()
    elif selection_text == 'Back to main menu':
        clear_screen()
        launch_main_menu()
        return

    if relaunch_wizard:
        clear_screen()
        launch_settings_menu()
    else:
        if get_confirmation("Would you like to continue with the demo?"):
            clear_screen()
            launch_settings_menu()
        else:
            print("Thank you for using Claudia demo. Bye!")
            return


def get_valid_repo_path():
    input_path = [
        inquirer.Text('repo path',
                      message="Please enter the absolute path to the rippled repo. Type 'back' or simply press ↵ (return) key to skip and go back to main menu.")
    ]
    full_repo_path = inquirer.prompt(input_path, theme=GreenPassion())['repo path']
    if full_repo_path == "" or full_repo_path == "back":
        pass
    else:
        if not os.path.isabs(full_repo_path) or not os.path.exists(full_repo_path):
            print(
                "The rippled repository path '{}' is not correct. Please provide correct absolute path!".format(
                    full_repo_path))
            return get_valid_repo_path()

    return full_repo_path

def get_rippled_branch():
    rippled_branch = inquirer.prompt(
        [inquirer.List("rippled_branch", message="Please choose rippled branch. Current Selection",
                       choices=["master", "develop", "release"], default="master")],
        theme=GreenPassion()
    )['rippled_branch']
    return rippled_branch


def get_testname():
    testname = [
        inquirer.Text('testname',
                      message="Please enter name test name. Press ↵ (return) key to include run everything. Type 'back' to skip and go back to main menu.")
    ]
    return inquirer.prompt(testname, theme=GreenPassion())['testname']


def get_feature_name():
    feature_name = [
        inquirer.Text('feature_name',
                      message="Please enter the feature name. Type 'back' or simply press ↵ (return) key to skip and go back to main menu.")
    ]
    return inquirer.prompt(feature_name, theme=GreenPassion())['feature_name']


def get_confirmation(confirmation_message):
    q = [inquirer.List("confirmation", message=confirmation_message, choices=["Yes", "No"], default="No")]
    answer = inquirer.prompt(q, theme=GreenPassion())['confirmation']
    if answer == 'Yes':
        return_value = True
    else:
        return_value = False
    return return_value


def clear_screen():
    if os.name == 'posix':
        os.system('clear')
    elif os.name == 'nt':
        os.system('cls')


@main.group()
def rippled():
    """Build or install rippled"""


@main.group()
def local_mainnet():
    """Setup rippled Network"""


@main.group()
def local_sidechain():
    """Setup Sidechain"""


@main.group()
def witness():
    """Setup Witness Server for Sidechain"""


@main.group()
def set_install_mode():
    """Setup Install Mode"""


@main.group()
@click.pass_context
def run(context):
    """Run XRPL automated tests"""


@main.command()
def clean():
    """Clean up the host and free resources"""
    launch_command_clean()


def launch_command_clean(ask_confirmation=True):
    if ask_confirmation:
        print("WARNING!! Running this command will wipe out everything built by Claudia. This action cannot be undone.")
        input("Press enter to continue or ctrl+c to cancel...")

    command = get_launch_command_clean()
    return subprocess.call(command, shell=True)


def get_launch_command_clean():
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --clean"


@main.command()
@click.option('--feature', required=True, help="The rippled feature name")
def enable_feature(feature):
    """Enable rippled feature"""
    launch_command_enable_feature(feature)


def launch_command_enable_feature(feature):
    command = get_launch_command_enable_feature(feature)
    return subprocess.call(command, shell=True)


def get_launch_command_enable_feature(feature):
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --enableFeature {}".format(feature)


@main.command()
@click.option('--feature', required=True, help="The rippled feature name")
def disable_feature(feature):
    """Disable rippled feature"""
    launch_command_disable_feature(feature)


def launch_command_disable_feature(feature):
    command = get_launch_command_disable_feature(feature)
    return subprocess.call(command, shell=True)


def get_launch_command_disable_feature(feature):
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --disableFeature {}".format(feature)


@main.group()
def list():
    """List supported options"""


@list.command()
def system_test_features():
    """List all supported features to be tested"""
    launch_command_system_test_features()


def launch_command_system_test_features():
    click.echo(get_launch_command_system_test_features_message())


def get_launch_command_system_test_features_message():
    set_to_project_root_wd()
    features = os.listdir('./features/')
    message = "Following features were found:\n"
    for i in range(0, len(features)):
        message += "   - {}\n".format(features[i].replace(".feature", ""))
    return message


@local_sidechain.command()
def start():
    """Start a new sidechain"""
    launch_command_sidechain_start()


def launch_command_sidechain_start():
    command = get_launch_command_sidechain_start()
    return subprocess.call(command, shell=True)


def get_launch_command_sidechain_start():
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --sidechainStart"


@local_sidechain.command()
def stop():
    """Stop sidechain"""
    launch_command_sidechain_stop()


def launch_command_sidechain_stop():
    command = get_launch_command_sidechain_stop()
    return subprocess.call(command, shell=True)


def get_launch_command_sidechain_stop():
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --sidechainStop"


@witness.command()
@click.option('--repo', required=True, help="The path to a local witness server repo")
def build(repo):
    """Build witness server"""
    launch_command_witness_build(repo)


def launch_command_witness_build(repo):
    if not os.path.isabs(repo) or not os.path.exists(repo):
        click.echo(
            " - ERROR: The rippled repository path '{}' is not correct. Please provide correct absolute path!".format(repo))
        return

    command = get_launch_command_witness_build(repo)
    return subprocess.call(command, shell=True)


def get_launch_command_witness_build(repo):
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --witnessBuild --repo {}".format(repo)


@witness.command()
def status():
    """Witness server status"""
    launch_command_witness_status()


def launch_command_witness_status():
    command = get_launch_command_witness_status()
    return subprocess.call(command, shell=True)


def get_launch_command_witness_status():
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --witnessStatus"


def get_launch_command_deploy_network(count):
    set_to_project_root_wd()
    return f"sh aws/setup.sh --deployNetwork --nodeCount {count}"


@set_install_mode.command()
def build():
    """Set install mode to build"""
    launch_command_set_install_mode_to_build()


def launch_command_set_install_mode_to_build():
    command = get_launch_command_set_install_mode_to_build()
    return subprocess.call(command, shell=True)


def get_launch_command_set_install_mode_to_build():
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --setInstallMode build"


@set_install_mode.command()
def install():
    """Set install mode to install"""
    launch_command_set_install_mode_to_install()


def launch_command_set_install_mode_to_install():
    command = get_launch_command_set_install_mode_to_install()
    return subprocess.call(command, shell=True)


def get_launch_command_set_install_mode_to_install():
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --setInstallMode install"


@list.command()
def system_requirements():
    """List all system requirements before continuing further with claudia"""
    launch_command_system_requirements()


def launch_command_system_requirements():
    click.echo(get_launch_command_system_requirements_message())


def get_launch_command_system_requirements_message():
    message = """
    1. Common requirements:
        - Python3
        - pip
        - docker
    2. Pull down a fresh copy of rippled code base from https://github.com/XRPLF/rippled
    3. Optional: Following depedencies are only required if you intend to run Javascript tests:
        - node
        - npm
    
        More detailed information can be found under the 'General prerequisite' section here: https://pypi.org/project/claudia
    """
    return message


@rippled.command()
@click.option('--repo', required=True, help="The path to a local rippled repo")
def build(repo):
    """Build rippled from source"""
    launch_command_rippled_build(repo)


def launch_command_rippled_build(repo):
    if not os.path.isabs(repo) or not os.path.exists(repo):
        click.echo(
            " - ERROR: The rippled repository path '{}' is not correct. Please provide correct absolute path!".format(repo))
        return
    command = get_launch_command_rippled_build(repo)
    return subprocess.call(command, shell=True)


def get_launch_command_rippled_build(repo):
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --rippledBuild --repo {}".format(repo)


@rippled.command()
def version():
    """View rippled version info"""
    launch_command_rippled_version()


def launch_command_rippled_version():
    command = get_launch_command_rippled_version()
    return subprocess.call(command, shell=True)


def get_launch_command_rippled_version():
    set_to_project_root_wd()
    return "sh network_setup/setup.sh --rippledVersion"


@rippled.command()
@click.option('--rippled_branch', default='master', help="Source branch for rippled binaries")
def install(rippled_branch):
    """Install rippled packages"""
    launch_command_rippled_install(rippled_branch)


def launch_command_rippled_install(rippled_branch):
    if rippled_branch not in ['master', 'develop', 'release']:
        print(f"The rippled branch '{rippled_branch}' is not correct. Allowed values are master, develop and release (case-sensitive).")
        return
    command = get_launch_command_rippled_install(rippled_branch)
    return subprocess.call(command, shell=True)


def get_launch_command_rippled_install(rippled_branch):
    set_to_project_root_wd()
    return f"sh network_setup/setup.sh --rippledInstall --rippledBranch {rippled_branch}"


@local_mainnet.command()
def start():
    """Start a new rippled network"""
    launch_command_network_start()


def launch_command_network_start():
    command = get_launch_command_network_start()
    return subprocess.call(command, shell=True)


def get_launch_command_network_start():
    set_to_project_root_wd()
    return "sh ./network_setup/setup.sh --networkStart"


@local_mainnet.command()
def stop():
    """Stop rippled network"""
    launch_command_network_stop()


def launch_command_network_stop():
    command = get_launch_command_sidechain_stop()
    return subprocess.call(command, shell=True)


def get_launch_command_network_stop():
    set_to_project_root_wd()
    return "sh ./network_setup/setup.sh --networkStop"


@local_mainnet.command()
def status():
    """rippled network status"""
    launch_command_network_status()


def launch_command_network_status():
    command = get_launch_command_network_status()
    return subprocess.call(command, shell=True)


def get_launch_command_network_status():
    set_to_project_root_wd()
    return "sh ./network_setup/setup.sh --networkStatus"


def get_explorer_url(network):
    url = ""
    if network == 'testnet':
        url = "https://testnet.xrpl.org"
    elif network == 'devnet':
        url = "https://devnet.xrpl.org"
    elif network == "local-sidechain":
        url = "https://custom.xrpl.org/localhost:6003"
    else:
        url = "https://custom.xrpl.org/localhost:6001"

    return url


@run.command()
@click.pass_context
@click.option('--lib', default='py',
              help="The type of client library to be used for running the tests. Allowed values are 'py' and 'js' and is defaulted to 'py'.  \n\nMore information: https://xrpl.org/client-libraries.html#client-libraries")
@click.option('--client_type', default='websocket',
              help="The type of client to be used. This flag should only be used with 'py' library. Allowed values are 'websocket' and 'jsonrpc' and is defaulted to 'websocket'.  \n\nMore information: https://xrpl.org/get-started-using-http-websocket-apis.html#differences-between-json-rpc-and-websocket")
@click.option('--network', default='local-mainnet',
              help="The type of network to be used. Allowed values are 'devnet', 'testnet', 'local-mainnet', and 'local-sidechain'; and is defaulted to 'local-mainnet'.  \n\nMore information: https://xrpl.org/get-started-using-http-websocket-apis.html#differences-between-json-rpc-and-websocket")
@click.option('--tag', default='smoke',
              help="Tag name of the all the tests to be included in the test run. Allowed values are 'smoke', 'regression' and 'time_intensive' and is defaulted to 'smoke'.  \n\nMore information: https://behave.readthedocs.io/en/latest/tag_expressions.html")
@click.option('--feature', default='payments',
              help=compose_feature_helper_message())
@click.option('--invalidate_cache', default='false',
              help="Forces ignoring cache, and reinstalling dependencies. This flag should only be used with 'js' library. Allowed values are 'true' and 'false' and is defaulted to 'false'.")
def systemtests(context, lib, client_type, network, tag, feature, invalidate_cache):
    """Launch XRPL system tests using XRPL client library"""
    launch_command_run_system_tests(lib, client_type, network, tag, feature, invalidate_cache)


def setup_system_test_env(lib, client_type, network, tag, feature, invalidate_cache):
    if lib == 'py':
        if invalidate_cache != 'false':
            raise Exception("--invalidate_cache flag is supported not with {} library client.".format(lib))
        set_to_py_wd()
        set_python_launch_vars(network, client_type)
        load_feature_files()
    elif lib == 'js':
        if client_type != 'websocket':
            raise Exception("Client Type {} is not supported with {} library client.".format(client_type, lib))
        set_to_js_wd()
        if invalidate_cache != 'false':
            command = "rm -rf ./node_modules"
            subprocess.call(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        install_js_dependencies()
        set_javascript_launch_vars(network)
        load_feature_files()
    else:
        raise Exception("Invalid library type: {}".format(lib))


def get_launch_command_system_test_tests(lib, tag, feature):
    if lib == 'py':
        if feature == "all":
            command = "behave --no-skipped --tags={}".format(tag)
        else:
            command = "behave --no-skipped --tags={} ./features/{}.feature".format(tag, feature)
    elif lib == 'js':
        if feature == "all":
            command = "npx cucumber-js --format @cucumber/pretty-formatter --tags @{}".format(tag)
        else:
            command = "npx cucumber-js --format @cucumber/pretty-formatter --tags @{} ./features/{}.feature".format(
                tag,
                feature
            )
    return command


def teardown_system_test_env(lib):
    if lib == 'py':
        unload_feature_files()
        set_to_project_root_wd()
    elif lib == 'js':
        unload_feature_files()
        set_to_project_root_wd()
    else:
        raise Exception("Invalid library type: {}".format(lib))


@run.command()
@click.option('--testname', default='everything',
              help="The unit test which needs to be selected. If not provided, all tests are selected.")
def unittests(testname):
    """Launch rippled unit tests"""
    launch_command_run_unit_tests(testname)


def launch_command_run_unit_tests(testname):
    command = get_launch_command_run_unit_tests(testname)
    return subprocess.call(command, shell=True)


def get_launch_command_run_unit_tests(testname):
    set_to_project_root_wd()
    if testname == '':
        return "sh network_setup/setup.sh --runUnittests"
    else:
        return "sh network_setup/setup.sh --runUnittests {}".format(testname)


def set_python_launch_vars(network, client_type):
    if network == "local-mainnet":
        if client_type == "websocket":
            connectionScheme = "ws"
            connectionURL = "127.0.0.1:6001"
            connectionType = "websocket"
        elif client_type == "jsonrpc":
            connectionScheme = "http"
            connectionURL = "127.0.0.1:5001"
            connectionType = "jsonrpc"
        else:
            raise Exception("{} is not a valid client_type".format(client_type))
    elif network == "local-sidechain":
        if client_type == "websocket":
            connectionScheme = "ws"
            connectionURL = "127.0.0.1:6003"
            connectionType = "websocket"
        elif client_type == "jsonrpc":
            connectionScheme = "http"
            connectionURL = "127.0.0.1:5003"
            connectionType = "jsonrpc"
        else:
            raise Exception("{} is not a valid client_type".format(client_type))
    elif network == "devnet":
        if client_type == "websocket":
            connectionScheme = "wss"
            connectionURL = "s.devnet.rippletest.net:51233"
            connectionType = "websocket"
        elif client_type == "jsonrpc":
            connectionScheme = "https"
            connectionURL = "s.devnet.rippletest.net:51234"
            connectionType = "jsonrpc"
        else:
            raise Exception("{} is not a valid client_type".format(client_type))
    elif network == "testnet":
        if client_type == "websocket":
            connectionScheme = "wss"
            connectionURL = "s.altnet.rippletest.net:51233"
            connectionType = "websocket"
        elif client_type == "jsonrpc":
            connectionScheme = "https"
            connectionURL = "s.altnet.rippletest.net:51234"
            connectionType = "jsonrpc"
        else:
            raise Exception("{} is not a valid client_type".format(client_type))
    else:
        raise Exception("{} is not a valid network".format(network))

    os.environ['CONNECTION_SCHEME'] = connectionScheme
    os.environ['CONNECTION_URL'] = connectionURL
    os.environ['CONNECTION_TYPE'] = connectionType


def set_javascript_launch_vars(network):
    if network == "local-mainnet":
        connectionScheme = "ws"
        connectionURL = "127.0.0.1:6001"
        connectionType = "websocket"
    elif network == "local-sidechain":
        connectionScheme = "ws"
        connectionURL = "127.0.0.1:6003"
        connectionType = "websocket"
    elif network == "devnet":
        connectionScheme = "wss"
        connectionURL = "s.devnet.rippletest.net:51233"
        connectionType = "websocket"
    elif network == "testnet":
        connectionScheme = "wss"
        connectionURL = "s.altnet.rippletest.net:51233"
        connectionType = "websocket"
    else:
        raise Exception("{} is not a valid network".format(network))

    os.environ['CONNECTION_SCHEME'] = connectionScheme
    os.environ['CONNECTION_URL'] = connectionURL
    os.environ['CONNECTION_TYPE'] = connectionType


def load_feature_files():
    unload_feature_files()
    os.popen("cp -fr ../features/*.feature ./features")


def unload_feature_files():
    os.popen("rm -rf ./features/*.feature")


def launch_behave(tag, feature):
    if feature == "all":
        command = "behave --no-skipped --tags={}".format(tag)
    else:
        command = "behave --no-skipped --tags={} ./features/{}.feature".format(tag, feature)
    return subprocess.call(command, shell=True)


def install_js_dependencies():
    command = "sh ./runSetup"
    return subprocess.call(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)


def launch_cucumber(tag, feature):
    if feature == "all":
        command = "npx cucumber-js --format @cucumber/pretty-formatter --tags @{}".format(tag)
    else:
        command = "npx cucumber-js --format @cucumber/pretty-formatter --tags @{} ./features/{}.feature".format(tag,
                                                                                                                feature)
    return subprocess.call(command, shell=True)


if __name__ == '__main__':
    main(context, obj={})



import os
import time
from enum import Enum

from behave import *
from hamcrest import assert_that, equal_to, contains_string
from xrpl.account import get_balance
from xrpl.asyncio.transaction import XRPLReliableSubmissionException
from xrpl.models import IssuedCurrencyAmount, PathStep, AccountInfo
from xrpl.transaction import submit_and_wait
from xrpl.wallet import Wallet
from claudia.python.lib.exceptions.InvalidInputException import InvalidInputException
from claudia.python.lib.exceptions.UnconfiguredEnvironmentException import UnconfiguredEnvironmentException
from claudia.python.lib.framework.object_factory import ObjFactory, ObjType
from claudia.python.lib.framework import constants


class ResponseCode(str, Enum):
    success = "tesSUCCESS"
    path_dry = "tecPATH_DRY"
    no_destination = "tecNO_DST"
    no_line_redundant = "tecNO_LINE_REDUNDANT"
    path_partial = "tecPATH_PARTIAL"
    unfunded_offer = "tecUNFUNDED_OFFER"
    insufficient_reserve = "tecINSUFFICIENT_RESERVE"
    no_permission = "tecNO_PERMISSION"
    has_obligations = "tecHAS_OBLIGATIONS"
    no_entry = "tecNO_ENTRY"
    unfunded_payment = "tecUNFUNDED_PAYMENT"


@given('we create "{account_type}" account for "{username} with "{amount}" XRP drops')
def step_impl(context, account_type, username, amount):
    account_type = sanitize_args(account_type)
    username = sanitize_args(username)
    if amount == "default":
        account_fund_amount = constants.DEFAULT_ACCOUNT_FUND_AMOUNT
    else:
        account_fund_amount = str(amount)

    # TODO: Change the account creation method to store in a collection
    if account_type == "source":
        context.sourceAccountUsername = username
        context.sourceWallet, context.sourceAddress, context.sourceBalance = generate_wallet_info(context,
                                                                                                  fund=True,
                                                                                                  fund_amount=account_fund_amount)
    elif account_type == "destination":
        context.destinationAccountUsername = username
        context.destinationWallet, context.destinationAddress, context.destinationBalance = generate_wallet_info(
            context)
    elif account_type == "issuer":
        context.issuerAccountUsername = username
        context.issuerWallet, context.issuerAddress, context.issuerBalance = generate_wallet_info(context)
    elif account_type == "generic_alice":
        context.aliceAccountUsername = username
        context.aliceWallet, context.aliceAddress, context.aliceBalance = generate_wallet_info(context)
    elif account_type == "generic_bob":
        context.bobAccountUsername = username
        context.bobWallet, context.bobAddress, context.bobBalance = generate_wallet_info(context)
    elif account_type == "generic_carol":
        context.carolAccountUsername = username
        context.carolWallet, context.carolAddress, context.carolBalance = generate_wallet_info(context)
    elif account_type == "invalid":
        context.invalidAccountUsername = username
        context.invalidWallet, context.invalidAddress, context.invalidBalance = generate_wallet_info(context)
        context.invalidAddress = "invalid"
        context.invalidBalance = 0
    else:
        raise InvalidInputException(
            "'{}' is not a valid accountType. Valid options are: 'source', 'destination' and 'issuer'.".format(
                account_type))


def generate_wallet_info(context, fund=True, fund_amount=constants.DEFAULT_ACCOUNT_FUND_AMOUNT):
    wallet = Wallet.create(algorithm=context.algorithm)
    if fund:
        send_payment(context, source_wallet=context.test_genesis_wallet, destination_wallet=wallet,
                     amount=fund_amount)
    return wallet, wallet.classic_address, get_balance(wallet.classic_address, context.client)


def send_payment(context, source_wallet, destination_wallet, amount):
    for i in range(0, 5):
        try:
            response = sign_autofill_and_submit(context.client,
                                                ObjFactory.getObj(
                                                    ObjType.payment,
                                                    account=source_wallet.classic_address,
                                                    destination=destination_wallet.classic_address,
                                                    amount=amount,
                                                    sequence=get_account_sequence(context.client,
                                                                                  source_wallet.classic_address)
                                                ),
                                                source_wallet)
        except XRPLReliableSubmissionException:
            time.sleep(1)
            continue
        break
    return response


def get_account_sequence(client, account_address):
    res = client.request(AccountInfo(
        account=account_address
    ))
    return res.result["account_data"]["Sequence"]


def wait_for_ledger_to_advance_for_account_delete(context, account_address, transaction_count=256):
    print("\tWaiting for ledger advance...")
    advance_ledger_with_transactions(context, transaction_count)
    count = 1
    account_1_sequence = ledger_sequence = 0
    while (ledger_sequence - account_1_sequence) <= transaction_count:
        account_1_sequence = get_account_sequence(context.client, account_address)
        ledger_sequence = get_ledger_current_index(context)
        if (count % 60) == 1:
            time.sleep(1)
        count += 1
    print("\tLedger advance complete...")


def get_ledger_current_index(context):
    ledger_current_request = ObjFactory.getObj(
        ObjType.ledger_current
    )
    return context.client.request(ledger_current_request).result['ledger_current_index']


def get_token_offers_response(client, account_id, token_id=None, offer_type=None):
    account_objects_response = get_account_objects_response(client, account_id)
    ledger_indexes = []
    for account_object in account_objects_response.result["account_objects"]:
        if account_object["LedgerEntryType"] == "NFTokenOffer":
            ledger_index = account_object["index"]
            if token_id:
                if account_object["NFTokenID"] == token_id:
                    if offer_type is None:
                        ledger_indexes.append(ledger_index)
                    else:
                        if offer_type == account_object["Flags"]:
                            ledger_indexes.append(ledger_index)
            else:
                if offer_type is None:
                    ledger_indexes.append(ledger_index)
                else:
                    if offer_type == account_object["Flags"]:
                        ledger_indexes.append(ledger_index)

    if token_id and len(ledger_indexes) == 0:
        raise Exception("Ledger Index not found for token '{}'".format(token_id))
    return ledger_indexes



def get_account_balance(context, account_id):
    if not account_id:
        raise Exception("Cannot get account balance. Account ID is invalid.")
    else:
        return get_balance(account_id, context.client)


def verify_test_status(actual_status, expected_status):
    assert_that(actual_status, equal_to(expected_status),
                "Test Status is incorrect. \nExpected: " + expected_status + "\nActual: " + actual_status)


def verify_test_error_message(actual_error, expected_error):
    assert_that(actual_error, contains_string(expected_error),
                "Incorrect error message received. \nExpected: " + expected_error + "\nActual: " + actual_error)


def advance_ledger_with_transactions(context, transaction_count):
    account_1, account_1_address, account_1_balance = generate_wallet_info(context)
    account_2, account_2_address, account_2_balance = generate_wallet_info(context)

    payment = ObjFactory.getObj(
        ObjType.payment,
        account=account_1_address,
        destination=account_2_address,
        amount="10"
    )
    print("\tAdvancing the ledger with " + str(transaction_count) + " transactions... This will take a while.")
    for i in range(0, transaction_count + 1):
        try:
            submit_and_wait(payment, context.client, account_1)
            print("Transaction # " + str(i) + ": succeeded")
        except KeyError:
            print("Transaction # " + str(i) + ": failed")


def get_nft_tokens(client, account_address):
    account_nfts = get_account_nfts(client, account_address).result["account_nfts"]
    nft_tokens = []
    for account_nft in account_nfts:
        nft_token = account_nft['NFTokenID']
        nft_tokens.append(nft_token)
    return nft_tokens


def get_ticket_sequence(client, account_address):
    account_objects = get_account_objects_response(client, account_address).result["account_objects"]
    ticket_sequences = []
    for account_object in account_objects:
        if account_object["LedgerEntryType"] == "Ticket":
            ticket_sequences.append(account_object['TicketSequence'])
    return ticket_sequences


def generate_issued_currency_amount(value, issuer, currency="USD"):
    return IssuedCurrencyAmount(
        value=value,
        issuer=issuer,
        currency=currency
    )


def generate_path_step(issuer=None, currency=None):
    if issuer is not None and currency is not None:
        return PathStep(
            issuer=issuer,
            currency=currency
        )
    if issuer is not None:
        return PathStep(
            issuer=issuer
        )
    if currency is not None:
        return PathStep(
            currency=currency
        )


def sign_autofill_and_submit(client, payload, wallet):
    return submit_and_wait(payload, client, wallet)


def verify_payment_account_objects(context, response, engine_result=ResponseCode.success):
    verify_not_creating_objects_type_account_objects(context, response, engine_result)


def verify_account_set_account_objects(context, response, engine_result=ResponseCode.success):
    verify_not_creating_objects_type_account_objects(context, response, engine_result)


def verify_not_creating_objects_type_account_objects(context, response, engine_result=ResponseCode.success):
    verify_transaction_validity(context, response, engine_result=engine_result)
    verify_response_code(response, engine_result)
    try:
        account_id = response.result["Owner"]
    except KeyError:
        account_id = response.result["Account"]

    transaction_type = response.result["TransactionType"]
    account_objects_response = get_account_objects_response(context.client, account_id)
    if account_objects_response.status != 'error':
        for account_object in account_objects_response.result["account_objects"]:
            assert transaction_type not in account_object['LedgerEntryType'], \
                "Account object created for Transaction Type: {}".format(transaction_type)


def verify_response_code(response, expected_code):
    assert expected_code in str(response), "Expected response code was not received. \nExpected Code = '{}'. " \
                                           "\nResponse received: '{}'".format(expected_code, response)


def verify_trustline_account_objects(context, response, engine_result=ResponseCode.success):
    verify_transaction_validity(context, response, engine_result=engine_result)
    verify_response_code(response, engine_result)

    try:
        account_id = response.result["Owner"]
    except KeyError:
        account_id = response.result["Account"]

    transaction_type = response.result["TransactionType"]
    if (transaction_type == "TrustSet"):
        account_objects_response = get_account_objects_response(context.client, account_id)
        if account_objects_response.status != 'error':
            assert account_id == account_objects_response.result['account']
            for account_object in account_objects_response.result["account_objects"]:
                assert (account_object['LedgerEntryType'] == "RippleState" or account_object[
                    'LedgerEntryType'] == 'Offer')


def verify_account_balance(actual_amount, expected_amount):
    assert actual_amount == expected_amount, "Account balance is incorrect. Expected: '{}' Actual: '{}'".format(
        expected_amount, actual_amount)


def verify_nftoken_mint_account_objects(context, response, engine_result=ResponseCode.success):
    verify_transaction_validity(context, response, engine_result=engine_result)
    verify_response_code(response, engine_result)
    try:
        account_id = response.result["Owner"]
    except KeyError:
        account_id = response.result["Account"]
    transaction_type = response.result["TransactionType"]
    hash_from_previous_txn = response.result["hash"]
    perform_account_object_verification = True

    if "engine_result_code" in response.result and response.result["engine_result_code"] != 0 and response.result[
        "engine_result_code"] != 150:  # OfferCreate tecKILLED flag
        perform_account_object_verification = False
    elif response.status != "success" or "tx_json" not in str(response):
        perform_account_object_verification = False

    if perform_account_object_verification:
        if transaction_type == "NFTokenMint":
            nft_tokens = get_nfts(context, account_id)
            assert nft_tokens, "nft_tokens not found"

            non_fungible_tokens = []
            tx_response = get_tx_response(context, hash_from_previous_txn)
            for affected_node in tx_response.result["meta"]["AffectedNodes"]:
                try:
                    if "CreatedNode" in affected_node and affected_node["CreatedNode"][
                        "LedgerEntryType"] == "NFTokenPage":
                        non_fungible_tokens_object = affected_node["CreatedNode"]["NewFields"]["NFTokens"]
                        assert len(non_fungible_tokens_object) <= constants.MAX_NFTOKEN_PAGE_OBJECTS_LIMIT, \
                            "NFToken page objects more than {} [{}]".format(
                                constants.MAX_NFTOKEN_PAGE_OBJECTS_LIMIT,
                                len(non_fungible_tokens_object))

                        for item in non_fungible_tokens_object:
                            non_fungible_token = item["NFToken"]["NFTokenID"]
                            non_fungible_tokens.append(non_fungible_token)

                    if "ModifiedNode" in affected_node and affected_node["ModifiedNode"][
                        "LedgerEntryType"] == "NFTokenPage":
                        non_fungible_tokens_object = affected_node["ModifiedNode"]["FinalFields"]["NFTokens"]
                        assert len(non_fungible_tokens_object) <= constants.MAX_NFTOKEN_PAGE_OBJECTS_LIMIT, \
                            "NFToken page objects more than {} [{}]".format(
                                constants.MAX_NFTOKEN_PAGE_OBJECTS_LIMIT,
                                len(non_fungible_tokens_object))

                        for item in non_fungible_tokens_object:
                            non_fungible_token = item["NFToken"]["NFTokenID"]
                            non_fungible_tokens.append(non_fungible_token)

                except KeyError as e:
                    pass

            if len(nft_tokens) <= constants.MAX_NFTOKEN_PAGE_OBJECTS_LIMIT:
                assert set(non_fungible_tokens) == set(nft_tokens), "Non Fungible Token object count mismatch"


def verify_nftoken_burn_account_objects(context, response, engine_result=ResponseCode.success):
    verify_transaction_validity(context, response, engine_result=engine_result)
    verify_response_code(response, engine_result)

    try:
        account_id = response.result["Owner"]
    except KeyError:
        account_id = response.result["Account"]
    hash_from_previous_txn = response.result["hash"]
    transaction_type = response.result["TransactionType"]
    object_removed = False
    account_objects_response = get_account_objects_response(context.client, account_id)

    if account_objects_response.status != 'error':
        if not account_objects_response.result["account_objects"] and transaction_type != "AccountDelete":
            object_removed = True
            deleted_node_found = True
        else:
            tx_response = get_tx_response(context, hash_from_previous_txn)
            for affected_node in tx_response.result["meta"]["AffectedNodes"]:
                try:
                    if "DeletedNode" in affected_node and transaction_type == tx_response.result["TransactionType"]:
                        deleted_node_found = True
                        ledger_index = affected_node["DeletedNode"]["LedgerIndex"]
                        previous_txn_id = affected_node["DeletedNode"]["FinalFields"]["PreviousTxnID"]
                        break
                except KeyError as e:
                    pass

            for account_object in account_objects_response.result["account_objects"]:
                if account_object["index"] == ledger_index or account_object["PreviousTxnID"] == previous_txn_id:
                    if transaction_type == "OfferCancel" and account_object["LedgerEntryType"] == "RippleState":
                        break
                    else:
                        account_object_found = True
                        break

            if previous_txn_id and not account_object_found:
                object_removed = True

        if not object_removed and not deleted_node_found and not account_object_found:
            assert previous_txn_id is None, "DeletedNode created for transaction type: {}".format(
                transaction_type)
        else:
            assert object_removed, "account object not cleared for txn '{}'".format(hash_from_previous_txn)


def verify_ticket_create_account_objects(context, response, engine_result=ResponseCode.success):
    verify_transaction_validity(context, response, engine_result=engine_result)
    verify_response_code(response, engine_result)

    try:
        account_id = response.result["Owner"]
    except KeyError:
        account_id = response.result["Account"]
    transaction_type = response.result["TransactionType"]

    if "engine_result_code" in response.result and response.result["engine_result_code"] != 0 and response.result[
        "engine_result_code"] != 150:
        perform_account_object_verification = False
    elif response.status != "success" or "tx_json" not in str(response):
        perform_account_object_verification = False
    if perform_account_object_verification:
        account_objects_response = get_account_objects_response(context.client, account_id)
        if account_objects_response.status != 'error':
            assert account_id == account_objects_response.result['account']
            for account_object in account_objects_response.result["account_objects"]:
                assert account_object['LedgerEntryType'] in transaction_type


def verify_account_delete_account_objects(context, response, engine_result=ResponseCode.success):
    verify_transaction_validity(context, response, engine_result=engine_result)
    verify_response_code(response, engine_result)

    try:
        account_id = response.result["Owner"]
    except KeyError:
        account_id = response.result["Account"]
    hash_from_previous_txn = response.result["hash"]
    transaction_type = response.result["TransactionType"]
    object_removed = False
    account_objects_response = get_account_objects_response(context.client, account_id)
    if transaction_type == "AccountDelete":
        if "error" in account_objects_response.result and account_objects_response.result["error"] == "actNotFound":
            object_removed = True
        assert object_removed, "account object not cleared for txn '{}'".format(hash_from_previous_txn)
        if account_objects_response.status != 'error':
            assert account_id == account_objects_response.result['account']
            for account_object in account_objects_response.result["account_objects"]:
                assert account_object['LedgerEntryType'] == "RippleState"


def verify_offer_create_account_objects(context, response, offer_crossing=None, engine_result=ResponseCode.success):
    verify_transaction_validity(context, response, engine_result=engine_result)
    verify_response_code(response, engine_result)
    perform_account_object_verification = True

    if "engine_result_code" in response.result and response.result["engine_result_code"] != 0 and response.result[
        "engine_result_code"] != 150:
        perform_account_object_verification = False
    elif response.status != "success" or "tx_json" not in str(response):
        perform_account_object_verification = False

    if perform_account_object_verification:
        try:
            account_id = response.result["Owner"]
        except KeyError:
            account_id = response.result["Account"]

        transaction_type = response.result["TransactionType"]
        if transaction_type == "OfferCreate":
            account_objects_response = get_account_objects_response(context.client, account_id)
            if account_objects_response.status != 'error':
                assert account_objects_response, "Account object not created in ledger"
                assert response.result["Account"] == account_objects_response.result[
                    'account'], "Account name is not correct"

                offer_crossing_status = False
                for account_object in account_objects_response.result["account_objects"]:
                    if offer_crossing:
                        if account_object['LedgerEntryType'] == "RippleState":
                            offer_crossing_status = True
                    elif offer_crossing is False:
                        if account_object['LedgerEntryType'] == "Offer":
                            offer_crossing_status = True
                    else:
                        if account_object['LedgerEntryType'] == "Offer":
                            offer_crossing_status = True

                assert offer_crossing_status


def get_account_objects_response(client, account_id):
    max_timeout = 20
    start_time = time.time()
    end_time = start_time + max_timeout
    account_objects = None
    while time.time() <= end_time and not account_objects:
        time.sleep(1)
        account_objects_request = ObjFactory.getObj(
            ObjType.account_object,
            account=account_id,
        )
        account_objects = (client.request(account_objects_request))
    return account_objects


def get_account_lines_response(context, account_id, ledger_index="validated"):
    account_lines_request = ObjFactory.getObj(
        ObjType.account_lines,
        account=account_id,
        ledger_index=ledger_index
    )
    return context.client.request(account_lines_request)


def get_tx_response(context, tx_id):
    tx_request = ObjFactory.getObj(
        ObjType.tx,
        transaction=tx_id,
        binary=False
    )
    return context.client.request(tx_request)


def get_nfts(context, account_id, ledger_index="validated"):
    account_nfts_request = ObjFactory.getObj(
        ObjType.account_nfts,
        account=account_id,
        ledger_index=ledger_index
    )
    account_nfts = context.client.request(account_nfts_request)
    nfts = []
    for item in account_nfts["account_nfts"]:
        nft_token = item['NFTokenID']
        nfts.append(nft_token)
    return nfts


def get_account_nfts(client, account_id, ledger_index="validated"):
    account_nfts_request = ObjFactory.getObj(
        ObjType.account_nfts,
        account=account_id,
        ledger_index=ledger_index
    )
    return client.request(account_nfts_request)


def sanitize_args(arg):
    if arg.startswith('"') and (arg.endswith('"')):
        fixed_arg = arg[1:]
        fixed_arg = fixed_arg[:-1]
        return fixed_arg
    else:
        return arg


def verify_transaction_validity(context, response=None, tx_id=None, engine_result="tesSUCCESS", max_timeout=30):
    is_tx_valid = False
    perform_txn_validation = False
    if tx_id:
        perform_txn_validation = True
    elif response:
        tx_id = response.result["hash"]
        if "engine_result" in response.result:
            if response.result["engine_result"] == "tesSUCCESS" or response.result["engine_result"] == "terQUEUED" or \
                    response["engine_result"] == "tecKILLED":
                perform_txn_validation = True
        elif response.status == "success":
            perform_txn_validation = True

    if perform_txn_validation:
        start_time = time.time()
        end_time = start_time + max_timeout
        transaction_result = None
        while time.time() <= end_time:
            tx_response = get_tx_response(context, tx_id)
            if "validated" in tx_response.result:
                if "meta" in tx_response.result:
                    transaction_result = tx_response.result["meta"]["TransactionResult"]
                if tx_response.result["validated"]:
                    if transaction_result == engine_result:
                        is_tx_valid = True
                        break
            time.sleep(1)

    if not is_tx_valid:
        return False


def read_env_var(key):
    if not os.environ.get(key, None):
        raise UnconfiguredEnvironmentException("'{}' is not set correctly".format(key))
    else:
        return os.environ[key]



from behave import *
from claudia.python.lib.framework.common import *
from claudia.python.lib.framework.object_factory import ObjFactory, ObjType
from xrpl.account import get_balance
from xrpl.models import IssuedCurrencyAmount, DepositPreauth, AccountSet, AccountSetAsfFlag
from xrpl.models.requests import Tx
from xrpl.transaction import submit


@when('we send a payment of "{transfer_amount}" XRP from Alice to Bob')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = float(transfer_amount) if '.' in transfer_amount else int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=str(context.transferAmount)
        )
        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP from Alice to self')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.sourceAddress,
            amount=str(context.transferAmount)
        )
        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we only submit a payment of "{transfer_amount}" XRP from Alice to Bob')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=str(context.transferAmount)
        )
        context.signed_payment_tx = submit(context.payment, context.sourceWallet)
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment with amount greater than balance from Alice to Bob')
def step_impl(context):
    try:
        context.transferAmount = get_balance(context.sourceAddress, context.client) + 1
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=str(context.transferAmount)
        )

        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP from Alice to Invalid')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.invalidAddress,
            amount=str(context.transferAmount)
        )

        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP to Bob without providing source information')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=str(context.transferAmount)
        )
        context.payment_response = submit_and_wait(context.payment, context.client, "")
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP from Alice without providing destination information')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=str(context.transferAmount)
        )

        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount_in_usd}" USD from Alice to Bob')
def step_impl(context, transfer_amount_in_usd):
    try:
        context.transferAmount = IssuedCurrencyAmount(
            currency="USD",
            value=str(transfer_amount_in_usd),
            issuer="rf1BiGeXwwQoi8Z2ueFYTEXSwuJYfV2Jpn"
        )

        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=context.transferAmount
        )

        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP with destination tag "{destination_tag}" info from Alice to Bob')
def step_impl(context, transfer_amount, destination_tag):
    try:
        context.transferAmount = int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            destination_tag=int(destination_tag),
            amount=str(context.transferAmount)
        )

        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP with invoice id "{invoice_id}" from Alice to Bob')
def step_impl(context, transfer_amount, invoice_id):
    try:
        context.transferAmount = int(transfer_amount)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            invoice_id=invoice_id,
            amount=str(context.transferAmount)
        )

        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP from Alice as a preauth account to Bob')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = int(transfer_amount)

        account_set_tx = AccountSet(
            account=context.destinationAddress,
            set_flag=AccountSetAsfFlag.ASF_DEPOSIT_AUTH,
        )

        submit_and_wait(account_set_tx, context.client, context.destinationWallet)

        deposit_pre_auth_tx = DepositPreauth(
            account=context.destinationAddress,
            sequence=get_account_sequence(context.client, context.destinationAddress),
            authorize=context.sourceAddress,
            fee=str(context.minDrop)
        )
        submit_and_wait(deposit_pre_auth_tx, context.client, context.destinationWallet)
        context.destinationBalance = get_balance(context.destinationAddress, context.client)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=str(context.transferAmount)
        )

        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@when('we send a payment of "{transfer_amount}" XRP from Alice as a non-preauth account to Bob')
def step_impl(context, transfer_amount):
    try:
        context.transferAmount = int(transfer_amount)
        account_set_tx = AccountSet(
            account=context.destinationAddress,
            set_flag=AccountSetAsfFlag.ASF_DEPOSIT_AUTH,
        )

        submit_and_wait(account_set_tx, context.client, context.destinationWallet)

        context.destinationBalance = get_balance(context.destinationAddress, context.client)
        context.payment = ObjFactory.getObj(
            ObjType.payment,
            account=context.sourceAddress,
            destination=context.destinationAddress,
            amount=str(context.transferAmount)
        )
        context.payment_response = submit_and_wait(context.payment, context.client, context.sourceWallet)
        context.response = context.client.request(Tx(transaction=context.payment_response.result["hash"]))
    except Exception as e:
        context.exception = e
        context.test_status = "failed"


@then('the balances after a valid payment are correct')
def step_impl(context):
    verify_payment_account_objects(context, context.response)
    verify_account_balance(get_account_balance(context, context.sourceAddress),
                           (context.sourceBalance - context.transferAmount - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress), (
            context.destinationBalance + context.transferAmount))


@then('the balances are correct in submit only mode')
def step_impl(context):
    verify_account_balance(get_account_balance(context, context.sourceAddress), (context.sourceBalance))
    verify_account_balance(get_account_balance(context, context.destinationAddress), (context.destinationBalance))


@then('the self-payment should fail')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception),
                              "An XRP payment transaction cannot have the same sender and destination.")
    verify_account_balance(get_account_balance(context, context.sourceAddress), context.sourceBalance)


@then('the payment with amount greater than balance should fail')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception),
                              "Transaction failed: tecUNFUNDED_PAYMENT")

    verify_account_balance(get_account_balance(context, context.sourceAddress),
                           (context.sourceBalance - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)


@then('the payment with invalid destination should fail')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception),
                              "Invalid value to construct an AccountID: expected valid classic address or "
                              "X-Address, received str.")
    verify_account_balance(get_account_balance(context, context.sourceAddress), context.sourceBalance)


@then('the payment with zero amount should fail')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception), "Can only send positive amounts.")
    verify_account_balance(get_account_balance(context, context.sourceAddress), context.sourceBalance)
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)


@then('the payment with decimal amount should fail')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception), "Error processing Amount: 10.5 is an invalid XRP amount.")
    verify_account_balance(get_account_balance(context, context.sourceAddress), context.sourceBalance)
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)


@then('the payment without source information should fail')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception), "Wallet must be provided when submitting an unsigned transaction")
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)


@then('the payment without destination information should fail')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception), "Destination not specified.")
    verify_account_balance(get_account_balance(context, context.sourceAddress), context.sourceBalance)


@then('the payment of 10 million XRP should fail')
def step_impl(context):
    verify_account_balance(get_account_balance(context, context.sourceAddress),
                           (context.sourceBalance - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)


@then('the balances after a non-xrp payment are correct')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception), "Transaction failed: tecPATH_DRY")
    verify_account_balance(get_account_balance(context, context.sourceAddress),
                           (context.sourceBalance - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)


@then('the balances after a payment with destination tag info are correct')
def step_impl(context):
    verify_payment_account_objects(context, context.response)
    verify_account_balance(get_account_balance(context, context.sourceAddress), (
            context.sourceBalance - context.transferAmount - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress),
                           (context.destinationBalance + context.transferAmount))


@then('the balances after a payment with invoice id info are correct')
def step_impl(context):
    verify_payment_account_objects(context, context.response)
    verify_account_balance(get_account_balance(context, context.sourceAddress),
                           (context.sourceBalance - context.transferAmount - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress),
                           (context.destinationBalance + context.transferAmount))


@then('the balances after a payment with invalid invoice id info are correct')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception), "non-hexadecimal number found in fromhex() arg at position 3")
    verify_account_balance(get_account_balance(context, context.sourceAddress), context.sourceBalance)
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)


@then('the balances after a payment from preauth account are correct')
def step_impl(context):
    verify_payment_account_objects(context, context.response)
    verify_account_balance(get_account_balance(context, context.sourceAddress),
                           (context.sourceBalance - context.transferAmount - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress),
                           (context.destinationBalance + context.transferAmount))


@then('the balances after a payment from non-preauth account are correct')
def step_impl(context):
    verify_test_status(context.test_status, "failed")
    verify_test_error_message(str(context.exception), "Transaction failed: tecNO_PERMISSION")
    verify_account_balance(get_account_balance(context, context.sourceAddress),
                           (context.sourceBalance - context.minDrop))
    verify_account_balance(get_account_balance(context, context.destinationAddress), context.destinationBalance)



import argparse
import glob
import json
import os
import pathlib
import requests
import sys
import time
from jinja2 import Environment, FileSystemLoader

file_path = pathlib.Path(__file__).parent.resolve()
GENESIS_ACCOUNT_ID = "rHb9CJAWyB4rj91VRWn96DkukG4bwdtyTh"
GENESIS_ACCOUNT_SEED = "snoPBrXtMeMyMHUVTgbuqAfg1SUTb"
DESTINATION_ACCOUNT_ID = "rh1HPuRVsYYvThxG2Bs1MfjmrVC73S16Fb"
DEFAULT_LOCKING_SERVER_ENDPOINT = "localhost:5001"
DEFAULT_ISSUING_SERVER_ENDPOINT = "localhost:5003"
DEFAULT_TRANSFER_AMOUNT = "1000000000"
MIN_ACCOUNT_CREATE_AMOUNT = "500000000"
DEFAULT_ACCOUNT_KEY_TYPE = "secp256k1"
WITNESS_SIGNING_KEY_KEY = "ed25519"
WITNESS_QUORUM = 0.8
VALIDATION_CHECK_TIMEOUT = 300  # max sec for validating tnx
SIGNATURE_REWARD = 100
SIDECHAIN_BRIDGE_CURRENCY_XRP = {"currency": "XRP"}
ISSUING_CHAIN_MASTER_ACCOUNT_DEFAULT_BALANCE = "1000000000000000"
NO_OF_WITNESS_SERVERS = 5
WITNESS_CONFIG_TEMPLATE_DIR = os.path.join(file_path, "../configs/sidechain/witness/template")


class RippledServer(object):
    def __init__(self, rippled_server):
        self.rippled_server = "http://{}".format(rippled_server)

    def execute_transaction(self, payload, method="submit", verbose=False):
        tx_json = None
        if "tx_json" in payload:
            tx_json = payload["tx_json"]
        if "secret" in payload:
            secret = payload["secret"]
            request = {"method": method, "params": [dict(tx_json=tx_json, secret=secret)]}
        else:
            request = {"method": method, "params": [tx_json]}

        try:
            response = requests.post(self.rippled_server, json.dumps(request))
            response_result = json.loads(response.content.decode('utf-8'))['result']

            if "engine_result" in response_result and (
                    response_result["engine_result"] == "tesSUCCESS" or
                    response_result["engine_result"] == "terQUEUED"):
                if self.is_transaction_validated(response_result) and verbose:
                    print("  Transaction validated!")

        except requests.exceptions.RequestException as e:
            print("Failed to establish connection to rippled")
            sys.exit(1)

        if verbose:
            print(response_result)

        if "engine_result_code" in response_result and response_result["engine_result_code"] != 0:
            print(response_result)
            sys.exit(1)
        return response_result

    def tx(self, tx_id):
        payload = {
            "tx_json": {
                "transaction": tx_id,
                "binary": False
            }
        }
        return self.execute_transaction(payload=payload, method="tx")

    def is_transaction_validated(self, response, engine_result="tesSUCCESS"):
        tx_id = response["tx_json"]["hash"]

        start_time = time.time()
        end_time = start_time + VALIDATION_CHECK_TIMEOUT
        while time.time() <= end_time:
            tx_response = self.tx(tx_id)
            if tx_response["validated"]:
                if response["engine_result"] == "terQUEUED":
                    return True
                else:
                    if tx_response["meta"]["TransactionResult"] == engine_result:
                        return True
            time.sleep(1)
        raise Exception("  Transaction not validated: {}".format(tx_id))

    def get_account_objects(self, account_id, verbose=False):
        payload = {
            "tx_json": {
                "account": account_id
            },
        }
        return self.execute_transaction(payload=payload, method="account_objects", verbose=verbose)

    def wallet_propose(self, seed=None, key_type=DEFAULT_ACCOUNT_KEY_TYPE):
        payload = {
            "tx_json": {
                "ledger_index": "current",
                "key_type": key_type
            }
        }
        if seed:
            payload["tx_json"]["seed"] = seed
        return self.execute_transaction(payload=payload, method="wallet_propose")

    def get_account_credentials(self, seed=None, key_type=DEFAULT_ACCOUNT_KEY_TYPE):
        wallet = self.wallet_propose(seed=seed, key_type=key_type)
        return wallet["account_id"], wallet["master_seed"]

    def make_payment(self, source_account_id=GENESIS_ACCOUNT_ID, source_account_seed=GENESIS_ACCOUNT_SEED,
                     destination_account_id=None, amount_to_transfer=MIN_ACCOUNT_CREATE_AMOUNT, verbose=False):
        if destination_account_id:
            if verbose:
                print(f"Transfer {amount_to_transfer} from {destination_account_id} to {source_account_id}")
            payload = {
                "tx_json": {
                    "TransactionType": "Payment",
                    "Account": source_account_id,
                    "Destination": destination_account_id,
                    "Amount": amount_to_transfer,
                },
                "secret": source_account_seed
            }
            return self.execute_transaction(payload=payload, method="submit", verbose=verbose)
        else:
            raise Exception("  Destination account cannot be None")

    def create_signer_list(self, signing_keys, signer_weight=1, verbose=False):
        signer_entries = []
        if verbose:
            print(f"Signing Keys: {signing_keys}")
        for signer_key in signing_keys:
            signer_entry = {
                "SignerEntry": {
                    "Account": signer_key,
                    "SignerWeight": signer_weight
                }
            }
            signer_entries.append(signer_entry)
        if verbose:
            print(f"signerEntries: {signer_entries}")
        return signer_entries

    def set_signer_list(self, account_id, seed, signer_entries, signer_quorum, verbose=False):
        if verbose:
            print("Set signer list set")

        payload = {
            "tx_json": {
                "Flags": 0,
                "TransactionType": "SignerListSet",
                "Account": account_id,
                "SignerQuorum": int(signer_quorum),
                "SignerEntries": signer_entries,
            },
            "secret": seed
        }
        return self.execute_transaction(payload=payload, method="submit")

    def create_xchain_bridge(self, bridge, door_account_id, door_account_seed, verbose=False):
        if verbose:
            print("Create bridge")

        payload = {
            "tx_json": {
                "Account": door_account_id,
                "TransactionType": "XChainCreateBridge",
                "XChainBridge": bridge,
                "SignatureReward": SIGNATURE_REWARD,
                "MinAccountCreateAmount": MIN_ACCOUNT_CREATE_AMOUNT
            },
            "secret": door_account_seed
        }
        self.execute_transaction(payload=payload, method="submit", verbose=verbose)

        for account_object in self.get_account_objects(door_account_id)["account_objects"]:
            if account_object["LedgerEntryType"] == "Bridge" and account_object["XChainBridge"] == bridge:
                return
        print("  XChain bridge creation failed!")
        sys.exit(1)

    def xchain_transfer(self, source_account_id=GENESIS_ACCOUNT_ID, source_account_seed=GENESIS_ACCOUNT_SEED,
                        destination_account_id=None, amount_to_transfer=DEFAULT_TRANSFER_AMOUNT,
                        bridge=None, verbose=False):
        print("- XChain Transfer")
        if bridge and destination_account_id:
            if verbose:
                print(f"Transfer {amount_to_transfer} from {destination_account_id} to {source_account_id}")
            payload = {
                "tx_json": {
                    "Account": source_account_id,
                    "Destination": destination_account_id,
                    "TransactionType": "XChainAccountCreateCommit",
                    "Amount": amount_to_transfer,
                    "SignatureReward": SIGNATURE_REWARD,
                    "XChainBridge": bridge
                },
                "secret": source_account_seed
            }
            return self.execute_transaction(payload=payload, method="submit", verbose=verbose)
        else:
            raise Exception("  bridge cannot be None")

    def get_account_balance(self, account_id):
        payload = {
            "tx_json": {
                "account": account_id,
                "ledger_index": "current",
                "strict": True,
                "queue": True
            }
        }

        count = 1
        max_retries = 10
        while count <= max_retries:
            account_info = self.execute_transaction(payload=payload, method="account_info")
            try:
                return int(account_info['account_data']['Balance'])
            except KeyError as e:
                count += 1
                time.sleep(2)
        return 0

    def disable_master_key_pair(self, key=GENESIS_ACCOUNT_ID, seed=GENESIS_ACCOUNT_SEED, verbose=False):
        if verbose:
            print("Disable Issuing chain Master key: {key}")

        payload = {
            "tx_json": {
                "TransactionType": "AccountSet",
                "Account": key,
                "SetFlag": 4
            },
            "secret": seed
        }
        return self.execute_transaction(payload=payload, method="submit", verbose=verbose)


def generate_witness_configs(rippled, bridge, witness_configs_dir):
    print("  Generate witness configs")
    signing_keys = []
    base_witness_rpc_endpoint_port = 6010

    file_loader = FileSystemLoader(WITNESS_CONFIG_TEMPLATE_DIR)
    env = Environment(loader=file_loader)
    template = env.get_template('witness.template')
    for count in range(1, NO_OF_WITNESS_SERVERS + 1):
        witness_name = f"witness{count:02d}"
        key, seed = rippled.get_account_credentials(key_type=WITNESS_SIGNING_KEY_KEY)
        locking_chain = {
            "Endpoint": {
                "Host": "rippled_1",
                "Port": "6006"
            },
            "TxnSubmit": {
                "SigningKeySeed": seed,
                "SigningKeyType": WITNESS_SIGNING_KEY_KEY,
                "SubmittingAccount": key
            },
            "RewardAccount": key
        }

        issuing_chain = {
            "Endpoint": {
                "Host": "rippled_3",
                "Port": "6006"
            },
            "TxnSubmit": {
                "SigningKeySeed": seed,
                "SigningKeyType": WITNESS_SIGNING_KEY_KEY,
                "SubmittingAccount": key
            },
            "RewardAccount": key
        }
        rpc_endpoint = {
            "Port": base_witness_rpc_endpoint_port + count
        }

        signing_key, signing_key_seed = rippled.get_account_credentials(key_type=WITNESS_SIGNING_KEY_KEY)
        signing_keys.append(signing_key)
        json_bridge = json.dumps(bridge, indent=2)

        content = template.render(locking_chain=locking_chain,
                                  issuing_chain=issuing_chain,
                                  rpc_endpoint=rpc_endpoint,
                                  witness_name=witness_name,
                                  xchain_bridge=json_bridge,
                                  signing_key_seed=signing_key_seed,
                                  signing_key_type=WITNESS_SIGNING_KEY_KEY)

        if not os.path.exists(witness_configs_dir):
            os.mkdir(witness_configs_dir)
        witness_config_file = os.path.join(witness_configs_dir, f"{witness_name}.json")
        with open(witness_config_file, mode="w") as output:
            output.write(content)

    return signing_keys


def setup_sidechain(locking_chain, issuing_chain, witness_configs_dir):
    print("- Setup Sidechain")
    locking_chain_door_account_id, locking_chain_door_seed = locking_chain.get_account_credentials()
    bridge = {
        "LockingChainDoor": locking_chain_door_account_id,
        "LockingChainIssue": SIDECHAIN_BRIDGE_CURRENCY_XRP,
        "IssuingChainDoor": GENESIS_ACCOUNT_ID,
        "IssuingChainIssue": SIDECHAIN_BRIDGE_CURRENCY_XRP
    }
    signing_keys = generate_witness_configs(rippled=locking_chain, bridge=bridge, witness_configs_dir=witness_configs_dir)

    locking_chain_submitting_account_ids = []
    issuing_chain_submitting_account_ids = []
    locking_chain_door_account_id = None

    witness_config_files = glob.glob(os.path.join(witness_configs_dir, 'witness*.json'))
    no_of_witnesses = 0
    for witness_config_filename in witness_config_files:
        no_of_witnesses += 1
        witness_config_file = f"{witness_config_filename}"

        with open(witness_config_file, "r") as fp:
            witness_data = json.load(fp)

        locking_chain_submitting_account_ids.append(witness_data["LockingChain"]["TxnSubmit"]["SubmittingAccount"])
        issuing_chain_submitting_account_ids.append(witness_data["IssuingChain"]["TxnSubmit"]["SubmittingAccount"])
        locking_chain_door_account_id = witness_data["XChainBridge"]["LockingChainDoor"]

    print(f"  Create Locking Chain Door account: {locking_chain_door_account_id}")
    locking_chain.make_payment(destination_account_id=locking_chain_door_account_id)

    print("  Create Locking Chain Submission accounts")
    for locking_chain_submitting_account_id in locking_chain_submitting_account_ids:
        locking_chain.make_payment(destination_account_id=locking_chain_submitting_account_id)

    print("  Create Issuing Chain Submission accounts")
    for issuing_chain_submitting_account_id in issuing_chain_submitting_account_ids:
        issuing_chain.make_payment(destination_account_id=issuing_chain_submitting_account_id)

    print("  Create SignerList")
    signer_entries = locking_chain.create_signer_list(signing_keys)
    locking_chain.set_signer_list(account_id=locking_chain_door_account_id, seed=locking_chain_door_seed,
                                  signer_entries=signer_entries, signer_quorum=WITNESS_QUORUM * no_of_witnesses)
    issuing_chain.set_signer_list(account_id=GENESIS_ACCOUNT_ID, seed=GENESIS_ACCOUNT_SEED,
                                  signer_entries=signer_entries, signer_quorum=WITNESS_QUORUM * no_of_witnesses)

    print("  Create XChain bridge")
    locking_chain.create_xchain_bridge(bridge, locking_chain_door_account_id, locking_chain_door_seed)
    issuing_chain.create_xchain_bridge(bridge, GENESIS_ACCOUNT_ID, GENESIS_ACCOUNT_SEED)

    print(f"  Disable issuing chain master key pair: {GENESIS_ACCOUNT_ID}")
    issuing_chain.disable_master_key_pair(key=GENESIS_ACCOUNT_ID, seed=GENESIS_ACCOUNT_SEED)


def xchain_transfer(locking_chain, issuing_chain, witness_configs_dir):
    witness_config_files = glob.glob(os.path.join(witness_configs_dir, 'witness*.json'))

    for witness_config_filename in witness_config_files:
        witness_config_file = f"{witness_config_filename}"

        with open(witness_config_file, "r") as fp:
            witness_data = json.load(fp)
            xchain_bridge = witness_data["XChainBridge"]
            break

    current_balance = issuing_chain.get_account_balance(account_id=DESTINATION_ACCOUNT_ID)
    locking_chain.xchain_transfer(bridge=xchain_bridge, destination_account_id=DESTINATION_ACCOUNT_ID,
                                  amount_to_transfer=ISSUING_CHAIN_MASTER_ACCOUNT_DEFAULT_BALANCE)
    if issuing_chain.get_account_balance(account_id=DESTINATION_ACCOUNT_ID) != \
            current_balance + int(ISSUING_CHAIN_MASTER_ACCOUNT_DEFAULT_BALANCE):
        print("  XChain Transfer failed")
        sys.exit(1)
    print(f"  Account {DESTINATION_ACCOUNT_ID} balance: {issuing_chain.get_account_balance(account_id=DESTINATION_ACCOUNT_ID)}")


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument('--lockingChain', default=DEFAULT_LOCKING_SERVER_ENDPOINT,
                        help="Locking chain RPC endpoint (default: {})".format(DEFAULT_LOCKING_SERVER_ENDPOINT))
    parser.add_argument('--issuingChain', default=DEFAULT_ISSUING_SERVER_ENDPOINT,
                        help="Issuing chain RPC endpoint (default: {})".format(DEFAULT_ISSUING_SERVER_ENDPOINT))
    parser.add_argument("--witnessConfigDir", required=True,
                        help="Path to witness configs directory")
    parser.add_argument("--xChainTransfer", help="Submit a XChain Transfer", action='store_true')

    return parser.parse_args()


def main(locking_chain_end_point, issuing_chain_end_point, witness_configs_dir, submit_xchain_transfer):
    locking_chain = RippledServer(locking_chain_end_point)
    issuing_chain = RippledServer(issuing_chain_end_point)
    if submit_xchain_transfer:
        xchain_transfer(locking_chain, issuing_chain, witness_configs_dir)
    else:
        setup_sidechain(locking_chain, issuing_chain, witness_configs_dir)


if __name__ == '__main__':
    cmd_args = parse_arguments()
    main(cmd_args.lockingChain, cmd_args.issuingChain, cmd_args.witnessConfigDir, cmd_args.xChainTransfer)



from enum import Enum

from xrpl.models import Payment, TrustSet, AccountLines, AccountObjects, OfferCreate, NFTokenMint, NFTokenBurn, \
    TicketCreate, AccountDelete, AccountNFTs, Tx, Memo, LedgerCurrent, NFTokenCreateOffer, NFTokenAcceptOffer
from claudia.python.lib.exceptions.InvalidInputException import InvalidInputException


class ObjFactory(object):
    @staticmethod
    def getObj(obj_type, **kwargs):
        payload = {}
        if obj_type == ObjType.payment:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]

            if "destination" in kwargs:
                payload["destination"] = kwargs["destination"]

            if "amount" in kwargs:
                payload["amount"] = kwargs["amount"]

            if "invoice_id" in kwargs:
                payload["invoice_id"] = kwargs["invoice_id"]

            if "destination_tag" in kwargs:
                payload["destination_tag"] = kwargs["destination_tag"]
            if "flags" in kwargs:
                payload["flags"] = kwargs["flags"]
            if "paths" in kwargs:
                payload["paths"] = kwargs["paths"]
            if "send_max" in kwargs:
                payload["send_max"] = kwargs["send_max"]
            obj = Payment(**payload)
        elif obj_type == ObjType.trust_set:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "limit_amount" in kwargs:
                payload["limit_amount"] = kwargs["limit_amount"]
            obj = TrustSet(**payload)
        elif obj_type == ObjType.tx:
            if "transaction" in kwargs:
                payload["transaction"] = kwargs["transaction"]
            if "binary" in kwargs:
                payload["binary"] = kwargs["binary"]
            obj = Tx(**payload)
        elif obj_type == ObjType.account_object:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            else:
                raise KeyError("'account' is a required field to get Account Lines")
            if "peer" in kwargs:
                payload["peer"] = kwargs["peer"]
            if "ledger_hash" in kwargs:
                payload["ledger_hash"] = kwargs["ledger_hash"]
            if "ledger_index" in kwargs:
                payload["ledger_index"] = kwargs["ledger_index"]
            if "type" in kwargs:
                payload["type"] = kwargs["type"]
            if "deletion_blockers_only" in kwargs:
                payload["deletion_blockers_only"] = bool(kwargs["deletion_blockers_only"])
            if "limit" in kwargs:
                payload["limit"] = kwargs["limit"]
            if "marker" in kwargs:
                payload["marker"] = kwargs["marker"]
            obj = AccountObjects(**payload)
        elif obj_type == ObjType.ledger_current:
            obj = LedgerCurrent()
        elif obj_type == ObjType.account_lines:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            else:
                raise KeyError("'account' is a required field to get Account Lines")
            if "peer" in kwargs:
                payload["peer"] = kwargs["peer"]
            if "limit" in kwargs:
                payload["limit"] = kwargs["limit"]
            if "marker" in kwargs:
                payload["marker"] = kwargs["marker"]
            if "ledger_index" in kwargs:
                payload["ledger_index"] = kwargs["ledger_index"]
            if "ledger_hash" in kwargs:
                payload["ledger_hash"] = kwargs["ledger_hash"]
            obj = AccountLines(**payload)
        elif obj_type == ObjType.account_nfts:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            else:
                raise KeyError("'account' is a required field to get Account NFTs")
            if "ledger_hash" in kwargs:
                payload["ledger_hash"] = kwargs["ledger_hash"]
            if "limit" in kwargs:
                payload["limit"] = kwargs["limit"]
            if "marker" in kwargs:
                payload["marker"] = kwargs["marker"]
            obj = AccountNFTs(**payload)
        elif obj_type == ObjType.offer_create:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "taker_pays" in kwargs:
                payload["taker_pays"] = kwargs["taker_pays"]
            if "taker_gets" in kwargs:
                payload["taker_gets"] = kwargs["taker_gets"]
            obj = OfferCreate(**payload)
        elif obj_type == ObjType.nft_token_mint:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "issuer" in kwargs:
                payload["issuer"] = kwargs["issuer"]
            if "uri" in kwargs:
                payload["uri"] = kwargs["uri"]
            if "nftoken_taxon" in kwargs:
                payload["nftoken_taxon"] = kwargs["nftoken_taxon"]
            if "transfer_fee" in kwargs:
                payload["transfer_fee"] = kwargs["transfer_fee"]
            if "flags" in kwargs:
                payload["flags"] = kwargs["flags"]
            if "memos" in kwargs:
                payload["memos"] = kwargs["memos"]
            if "ticket_sequence" in kwargs:
                payload["ticket_sequence"] = kwargs["ticket_sequence"]
            if "sequence" in kwargs:
                payload["sequence"] = kwargs["sequence"]
            obj = NFTokenMint(**payload)
        elif obj_type == ObjType.nft_token_burn:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "fee" in kwargs:
                payload["fee"] = kwargs["fee"]
            if "owner" in kwargs:
                payload["owner"] = kwargs["owner"]
            if "nftoken_id" in kwargs:
                payload["nftoken_id"] = kwargs["nftoken_id"]
            obj = NFTokenBurn(**payload)
        elif obj_type == ObjType.ticket_create:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "sequence" in kwargs:
                payload["sequence"] = kwargs["sequence"]
            if "ticket_count" in kwargs:
                payload["ticket_count"] = kwargs["ticket_count"]
            obj = TicketCreate(**payload)
        elif obj_type == ObjType.memo:
            if "memo_data" in kwargs:
                payload["memo_data"] = kwargs["memo_data"]
            if "memo_format" in kwargs:
                payload["memo_format"] = kwargs["memo_format"]
            if "memo_type" in kwargs:
                payload["memo_type"] = kwargs["memo_type"]
            obj = Memo(**payload)
        elif obj_type == ObjType.account_delete:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "destination" in kwargs:
                payload["destination"] = kwargs["destination"]
            if "fee" in kwargs:
                payload["fee"] = kwargs["fee"]
            if "sequence" in kwargs:
                payload["sequence"] = kwargs["sequence"]
            obj = AccountDelete(**payload)
        elif obj_type == ObjType.nf_token_create_offer:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "destination" in kwargs:
                payload["destination"] = kwargs["destination"]
            if "owner" in kwargs:
                payload["owner"] = kwargs["owner"]
            if "nftoken_id" in kwargs:
                payload["nftoken_id"] = kwargs["nftoken_id"]
            if "flags" in kwargs:
                payload["flags"] = kwargs["flags"]
            if "expiration" in kwargs:
                payload["expiration"] = kwargs["expiration"]
            if "amount" in kwargs:
                payload["amount"] = kwargs["amount"]
            obj = NFTokenCreateOffer(**payload)
        elif obj_type == ObjType.nf_token_accept_offer:
            if "account" in kwargs:
                payload["account"] = kwargs["account"]
            if "nftoken_broker_fee" in kwargs:
                payload["nftoken_broker_fee"] = kwargs["nftoken_broker_fee"]
            if "nftoken_buy_offer" in kwargs:
                payload["nftoken_buy_offer"] = kwargs["nftoken_buy_offer"]
            if "nftoken_sell_offer" in kwargs:
                payload["nftoken_sell_offer"] = kwargs["nftoken_sell_offer"]
            obj = NFTokenAcceptOffer(**payload)
        else:
            raise InvalidInputException("'{}' is not a valid objectType.".format(obj_type))
        return obj


class ObjType(str, Enum):
    payment = "payment"
    trust_set = "trust_set"
    account_lines = "account_lines"
    account_object = "account_object"
    offer_create = "offer_create"
    nft_token_mint = "nft_token_mint"
    nft_token_burn = "nft_token_burn"
    ticket_create = "ticket_create"
    account_delete = "account_delete"
    tx = "tx"
    account_nfts = "account_nfts"
    memo = "memo"
    ledger_current = "ledger_current"
    nf_token_create_offer = "nf_token_create_offer"
    nf_token_accept_offer = "nf_token_accept_offer"




################################################################################
# This script is used to validate a rippled network by:
# 1. Verifying "server_state"
# 2. Perform a payment transaction
#
# Usage:
#   python3 scripts/validate_network.py [optional parameter]
#       [--rippledServer <rippled host:port (default: localhost:5001)>
################################################################################

import argparse
import json
import sys
import requests
import time

GENESIS_ACCOUNT_ID = "rHb9CJAWyB4rj91VRWn96DkukG4bwdtyTh"
GENESIS_ACCOUNT_SEED = "snoPBrXtMeMyMHUVTgbuqAfg1SUTb"
DEFAULT_RIPPLED_SERVER = "localhost:5001"
TRANSFER_AMOUNT = "10000000000"
DESTINATION_ACCOUNT_ID = "rh1HPuRVsYYvThxG2Bs1MfjmrVC73S16Fb"
VALIDATION_CHECK_TIMEOUT = 300  # seconds


class RippledServer(object):
    def __init__(self, rippled_server=DEFAULT_RIPPLED_SERVER):
        self.rippled_server = "http://{}".format(rippled_server)

    def execute_transaction(self, payload, method):
        tx_json = None
        if "tx_json" in payload:
            tx_json = payload["tx_json"]
        if "secret" in payload:
            secret = payload["secret"]
            request = {"method": method, "params": [dict(tx_json=tx_json, secret=secret)]}
        else:
            request = {"method": method, "params": [tx_json]}

        try:
            response = requests.post(self.rippled_server, json.dumps(request))
            response_result = json.loads(response.content.decode('utf-8'))['result']

            if "engine_result" in response_result and (
                    response_result["engine_result"] == "tesSUCCESS" or
                    response_result["engine_result"] == "terQUEUED"):
                if self.is_transaction_validated(response_result):
                    print("  Transaction validated!")

        except requests.exceptions.RequestException as e:
            print("Failed to establish connection to rippled")
            sys.exit(1)

        return response_result

    def tx(self, tx_id):
        payload = {
            "tx_json": {
                "transaction": tx_id,
                "binary": False
            }
        }
        return self.execute_transaction(payload=payload, method="tx")

    def get_account_balance(self, account_id):
        payload = {
            "tx_json": {
                "account": account_id,
                "ledger_index": "current",
                "strict": True,
                "queue": True
            },
        }
        account_info = self.execute_transaction(payload=payload, method="account_info")
        try:
            return int(account_info['account_data']['Balance'])
        except KeyError as e:
            return 0

    def is_transaction_validated(self, response, engine_result="tesSUCCESS"):
        tx_id = response["tx_json"]["hash"]

        VALIDATION_CHECK_TIMEOUT = 30  # max sec for validating tnx
        start_time = time.time()
        end_time = start_time + VALIDATION_CHECK_TIMEOUT
        while time.time() <= end_time:
            tx_response = self.tx(tx_id)
            if tx_response["validated"]:
                if response["engine_result"] == "terQUEUED":
                    return True
                else:
                    if tx_response["meta"]["TransactionResult"] == engine_result:
                        return True
            time.sleep(1)
        raise Exception("  Transaction not validated: {}".format(tx_id))


def verify_network(rippled):
    print("  Server State: ", end="")
    start_time = time.time()
    while True:
        time.sleep(5)
        server_info = rippled.execute_transaction(payload={}, method="server_info")
        server_state = server_info["info"]["server_state"]
        proposers = server_info["info"]["last_close"]["proposers"]
        if server_state in ("full", "proposing") and proposers != 0:
            print("{}".format(server_state))
            break

        if time.time() > (start_time + VALIDATION_CHECK_TIMEOUT):
            print("{}".format(server_state))
            sys.exit(1)


def verify_payment_txn(rippled):
    print("  Verify Payment Transaction")
    initial_balance = int(rippled.get_account_balance(DESTINATION_ACCOUNT_ID))

    payload = {
        "tx_json": {
            "TransactionType": "Payment",
            "Account": GENESIS_ACCOUNT_ID,
            "Destination": DESTINATION_ACCOUNT_ID,
            "Amount": TRANSFER_AMOUNT,
        },
        "secret": GENESIS_ACCOUNT_SEED
    }
    rippled.execute_transaction(payload=payload, method="submit")
    new_balance = int(rippled.get_account_balance(DESTINATION_ACCOUNT_ID))

    if new_balance != (initial_balance + int(TRANSFER_AMOUNT)):
        print("  Account balance mismatch")
        sys.exit(1)


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument('--rippledServer', default=DEFAULT_RIPPLED_SERVER,
                        help="rippled server (default: {})".format(DEFAULT_RIPPLED_SERVER))
    parser.add_argument("--paymentTransaction", help="Submit a Payment Transaction", action='store_true')

    return parser.parse_args()


def main(rippled_server, submit_payment_txn):
    rippled = RippledServer(rippled_server)
    verify_network(rippled)
    if submit_payment_txn:
        verify_payment_txn(rippled)


if __name__ == '__main__':
    cmd_args = parse_arguments()
    main(cmd_args.rippledServer, cmd_args.paymentTransaction)



DEFAULT_TRANSFER_AMOUNT = "1000"
DEFAULT_ACCOUNT_FUND_AMOUNT = "300000000"
MAX_NFTOKEN_PAGE_OBJECTS_LIMIT = 32
DEFAULT_TEST_GENESIS_ACCOUNT_FUND = "10000000000"
DEFAULT_DELETE_ACCOUNT_FEE = "2000000"
BASE_RESERVE = "10000000"
OWNER_RESERVE = "2000000"



import os

from src.claudia.versions import CLAUDIA_VERSION

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

current_directory = os.path.dirname(os.path.abspath(__file__))
try:
    with open(os.path.join(current_directory, 'src/claudia/README.md'), encoding='utf-8') as f:
        long_description = f.read()
except Exception:
    long_description = ''


def read_requirements():
    print(os.getcwd())
    with open('src/claudia/requirements.txt') as req:
        content = req.read()
        requirements = content.split('\n')

    return requirements


setup(
    name='claudia',
    version=CLAUDIA_VERSION,
    description='Run XRPL Automated Tests',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Kausty Saxena",
    author_email="ksaxena@ripple.com",
    keywords="ripple xrpl python javascript",
    url='https://xrpl.org/',
    download_url='https://gitlab.ops.ripple.com/xrpledger/xrpl-nocode-automation',
    py_modules=['claudia.claudia'],
    install_requires=read_requirements(),
    packages=['claudia'],
    package_dir={"claudia": "src/claudia"},
    package_data={'claudia': [
        "./requirements.txt",
        "./README.md",
        "./network_setup/**/*",
        "./features/*.feature",
        "./javascript/**/*",
        "./python/**/*",
        "./ui/**/*",
        "./.streamlit/**/*",
        "./aws/**/*"
    ]},
    include_package_data=True,
    entry_points='''
        [console_scripts]
        claudia=claudia.claudia:main
    '''
)
